# Generated by roxygen2: do not edit by hand

export(FR.test)
export(TS_cont_pval)
export(bincounterR)
export(case.studies)
export(change.marginals)
export(chiTS.cont)
export(chiTS.disc)
export(power_pvals)
export(rDalitz)
export(run.studies)
export(signif_digits)
export(simTS)
export(simpvals)
export(test_methods)
export(timecheck)
export(twosample_power)
export(twosample_test)
export(twosample_test_adjusted_pvalue)
importFrom(Rcpp,sourceCpp)
importFrom(stats,qexp)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rexp)
importFrom(stats,runif)
useDynLib(MD2sample, .registration=TRUE)
