#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom stats qexp
#' @importFrom stats qnorm
#' @importFrom stats rbeta
#' @importFrom stats rexp
#' @importFrom stats runif
## usethis namespace: end
NULL
