% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{calcTS}
\alias{calcTS}
\title{This function calculates the test statistics}
\usage{
calcTS(dta, TS, typeTS, TSextra)
}
\arguments{
\item{dta}{data set}

\item{TS}{routine}

\item{typeTS}{format of TS}

\item{TSextra}{list passed to TS function}
}
\value{
A vector of numbers
}
\description{
This function calculates the test statistics
}
\keyword{internal}
